// GRAPH.H
// Gregory Ecker
// Graph class version 1.0
// a Graph object stores arrays of vectors (points) so all matrix operations
// can be performed essentially the same as polygon objects.
// Graph also stores an array of edges which are indices into the point array.

#ifndef _GRAPH_H
#define _GRAPH_H
#include <assert.h>
#include <stdarg.h>

#include "commdefs.h"
#include "point.h"
#include "polygon.h"
#include "matrix.h"
#include "clipline.h"

// index into point array defining start,end vertices to draw line:
typedef struct {
	int from;
	int to;
	int color;
} edge;

class graph {
	public:
		graph();             // default constructor
		graph(int,int);      // number elt.s in vertex, edge arrays.
		graph(graph&);       // copy constructor.
		graph(const polygon&);  // make a graph from a polygon.
		graph(int,int,int,int); //variable sized max, num edges.pts.
	       ~graph();		// destructor;
void	Project(void);               // perform projection on each point.
void	Draw(void);		     // draw all edges in graph.
void 	Apply(matrix&);		     // apply matrix to each point in array.

// maintenance of edge array (in case edges are renamed).
// to be used in case of deleteion,insertion etc.
int 	Replace_edge_component(int,int,int,...);// old edge value, new edge value.

// point insertions, deletions, additions:
void	Insert_pt(vector* ptr,int); // insert point pointed to by ptr.
void    Delete_pt(int);         // delete point indexed into point array.
vector* Replace_pt(int,vector*); // replace index point with new point.
void    Add_pt(vector* ptr);	// add new point add end of list.
// edge delete, addition.
//edge*
void    Insert_edge(edge*,int); // inserts indexed edge into edge array.
void    Delete_edge(int index);	// delete edge indexed into edge array.
void    Add_edge(edge*);	// add new edge to edge list.

// memory management functions: (size of vertex,edge list)
//	all return 1 on success:
 int   enlarge_vertex_array_by(int enlargement);
 int   enlarge_vertex_array_to(int new_size);
 int   enlarge_edge_array_by(int enlargement);
 int   enlarge_edge_array_to(int new_size);

    // publicized private data functions:
inline int      Numpts(void) const { return numpts; };
inline int	Numedges(void) const { return numedges; };
inline int	Max_pts(void) const { return max_pts; };
inline int	Max_edges(void) const { return max_edges; };

// index functions into pt and edge array.
inline vector&  pt_array(int index) { assert((index>=0)&&(index<numpts));
				      return *pts[index]; };
inline vector&  operator[](int index){ assert((index>=0)&&(index<numpts));
				      return *pts[index]; };
inline edge&   edge_array(int index){assert((index>=0)&&(index<numedges));
				      return *edges[index]; };
    // public data
      BOOLEAN initialized;         // objects are not automatically init.
      BOOLEAN colored;
      int Tx,Ty;		   // offsets into device coordinates.
	private:
	 int numpts,numedges;      // number of pt.s,vertices in use.
	 int max_pts,max_edges;    // number of pt.s,vertices allocated.

	 vector **pts;		   // array of vector (points)
	 edge **edges;	   // array of edges (indices into pts array).
}; // end class graph.
#endif