// GRID.H
// Gregory Ecker
// 8/29/94
// notes:
// The grid object is used to select a point in 3d space with a 2d mouse.
// The points selectable by the mouse are the points of intersection of
// the lines in the grid.  The resolution (number of lines) of the grid
// can be adjusted as well as the scale factor of the units representing
// distance between grid intersections.
// The grid may be rotated, in which case all of the points maintain their
// proximity to each other.  The grid is a way of constructing objects,
// and each point,line entry will be saved into a graph object.
//#ifndef _GRID_H
//#define _GRID_H


#include "graph.h"
// default resolution: (x,y)
// note: this must be an even number!
#define DEF_GRID_X 20   // horizontal resolution.
#define DEF_GRID_Y 20   // vertical resolution.
#define UNIT_VALUE 10   // space between selectable points (scale).

class grid {
 public:
  grid();	// default constructor.

 private:
  int X_res,Y_res;
  float Zcoordinate;       // z coordinate of grid.
  graph *A_grid,*out_grid; // A_grid is selectable by the mouse,
			   // out_grid is the contruction so far.
};

/////////////////////////////////////////////////////////////////////////
// default constructor:
//	initializes resolution to default, sets up drawing apparatus
//	(A_grid), and allocates new graph object to store the construction.
grid::grid(){
     vector *tmp;
	Zcoordinate=1;     // starting zcoordinate of grid.
	X_res=DEF_GRID_X;
	Y_res=DEF_GRID_Y;
	float half_X_res_Scale=(X_res/2)*UNIT_VALUE; // offset from origin
	float half_Y_res_Scale=(Y_res/2)*UNIT_VALUE; // of outermost points.
	// instantiate new graph object with # of vertices, edges
	A_grid = new graph(X_res*Y_res,X_res+Y_res);
	// allocate memory for all vertices and edges and initialize them
	// with default 'grid' values.
    int index=0,edge_index=0;
    edge *temp1,*temp2;
  for(int i=0;i<X_res;i++){
   // vertical edges:
     temp1=new edge;
     temp1->from=i;
   for(int j=0;j<Y_res;j++){
     tmp = new vector(i*UNIT_VALUE-half_X_res_Scale,j*UNIT_VALUE-
	half_Y_res_Scale,Zcoordinate,1);
     A_grid->Insert_pt(tmp,index);
     index++;
    }
    temp1->to=index;
    A_grid->Insert_edge(temp1,edge_index);
   }
};
