
// Gregory Ecker
// Numerical Computing
// Prof Overton

// 'mat2x3dg.cpp'
// converts matlab vector data (13 rows, 1 col) into
// .3d file format containing the 3d vector description
// of a bridge.

// the magnitude of the forces are represented by the brightness
// of the lines representing the bars in the bridge.

// this file assumes the 3d file viewer (3dg) is envoked with
// the "b","g" or "r" option, thus putting it into 2-color mode,
// of 7 shades each.

#include <iostream.h>
#include <process.h>
#include <assert.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "commdefs.h"

#define DEBUG 1
#define NUMSHADES 7  // number of shades available for each color.
#define M 13	     // number of equations.

int whichIndex(int,int,int); // returns index of line segment being drawn.
			     // based upon vertex numbers.

void main(int argc,char* argv[]){
 FILE *sourceFile, *destFile , *templateFile;
 float negMax,negMin,posMax,posMin;
 char sourceFileName[12];
 char destFileName[12];
 char *templateFileName="bridge.3d"; // default template file.
 float solution[M]; // space for solution vector to be read from .mat file
 int shades[M];     // solution vector interpolated as shades of color.
 float tmp;
   if(argc != 3){
     cout << endl << "mat2x3dg usage summary:" << endl;
     cout << "converts ascii matlab solution vectors (13 x 1) "<< endl;
     cout << "into datafiles viewable with 3dg20 (c) EcktoSoft" << endl;
     cout << endl << "syntax : " << endl;
     cout << "    mat2x3dg  <sourcefile>.mat <destfile>.3d" << endl;
     exit(1);
   }

    // get name of source file:
     strcpy(sourceFileName,argv[1]);

    // open source file, if error, report + exit.
     sourceFile = NULL;
     if((sourceFile = fopen(sourceFileName,READ))==NULL){
	cout << "error opening " << sourceFileName << " try again " << endl;
	exit(1);
      };


 // read all elements of solution vector (13 elt.s) into double solution[13]
     for(int i=0;i<M;i++){
      fscanf(sourceFile, "%e", &tmp);
      solution[i]=tmp;
     }

 // close up file stream:
     fclose(sourceFile);


 // in order to find the proper pallete, we first sort the element from
 // the solution vector;
 float solution2[M]; // copy of original solution vector;
 for(i=0;i<M;i++){
    solution2[i]=solution[i];
  }

 float temp;
 // sort the copy of the solution vector:
  for(i=0;i<(M-1);i++){
    for (int j=M; j> (i+1);j--){
    if(solution2[j] <= solution2[j-1]) {
      temp = solution2[j];
      solution2[j] = solution2[j-1];
      solution2[j-1] = temp;
    }

   }
  }
 // now, to find the proper color for the force, find it's new index
 // into the new solution vector (solution2);


 int aShade=0; int numNegative=0; // numNegative to find out how many colors
 float oldValue;		  // needed for each (positive, negative force).
 for(i=0;i<M;i++){
    aShade =0;
    oldValue=solution[i];
   // go through newly sorted values:
    while(oldValue != solution2[aShade]){
      aShade++;
     };
    if(oldValue<0){
      numNegative ++;
     };
     shades[i]=aShade+1; // give edge it's brightness depending upon magnitude.
  };


/*  #ifdef DEBUG
   for(i=0;i<M;i++){
     cout << shades[i] << endl;
    }
  #endif */

    // get name of destination file to create, if exist, overwrite.
     strcpy(destFileName,argv[2]);

     if ((destFile = fopen(destFileName, "wt")) == NULL){
	cout << "Cannot open output file. exit(1) " << endl;
	exit(1);
     }
    // open up template file name with original bridge data in it:
     if((templateFile = fopen(templateFileName,READ)) == NULL){
	cout << "Cannot open temple .3d file: 'bridge.3d, exit(1) " << endl;
	exit(1);
     }

   // read scaling info, and echo to new file:
     int inttmp;
     fscanf(templateFile," %i ",&inttmp);
     fprintf(destFile," %i ",inttmp);
     fscanf(templateFile," %i ",&inttmp);
     fprintf(destFile," %i ",inttmp);
     fscanf(templateFile," %i",&inttmp);
     fprintf(destFile," %i \n",inttmp);

   // read number of points (same for both files)
     fscanf(templateFile,"%i",&inttmp);
     fprintf(destFile,"%i \n",inttmp);

   float floattemp,floattemp2,floattemp3;

    for(int k=0;k<inttmp;k++){ // all points are the same.
     fscanf(templateFile,"%f",&floattemp);  // x
     fscanf(templateFile,"%f",&floattemp2); // y
     fscanf(templateFile,"%f",&floattemp3); // z coordinates

     fprintf(destFile,"%f ",floattemp);
     fprintf(destFile,"%f ",floattemp2);
     fprintf(destFile,"%f \n",floattemp3);
    };

   // read number of lines (same for both files)
     fscanf(templateFile,"%i",&inttmp);
     fprintf(destFile,"%i \n",inttmp);


   // read all point definitions (colors must be changed);
   int temp1,temp2,newcolor,lastVertex,index;
   for(k=0;k<inttmp;k++){

     fscanf(templateFile,"%i",&temp1);
     fscanf(templateFile,"%i",&temp2);

     if(temp2==0){  // temp2 ==0 indicates a jump, and not a real line.
       fprintf(destFile,"%i ",temp1); // just echo it to new file.
       fprintf(destFile,"%i \n",temp2);
       lastVertex=temp1;
     } else {	// otherwise, the color must be changed, depending upon the
       index = whichIndex(lastVertex,temp1,0);  // line segment being drawn.

       if(index == 0){
	 index = whichIndex(lastVertex, temp1,8);
	};

       newcolor = shades[index % M]+1;

       fprintf(destFile,"%i ",temp1); // point data is the same.
       fprintf(destFile,"%i \n",newcolor); // color is different.
       lastVertex=temp1;
     };

   }



   // close up file streams:
     fclose(destFile);
     fclose(templateFile);
cout << "newly created file: " << destFileName << endl;
cout << "number of negative elements : " << numNegative << endl;
};


int whichIndex(int from,int to,int b){  // returns index of line segment being drawn.

    if((from == 1+b) && (to == 2+b)){
	return 1+b;
     } else
    if((from == 2+b) && (to == 4+b)){
	return 4+b;
     } else
    if((from==4+b) && (to == 6+b)){
	return 8+b;
     } else
    if((from == 6+b) && (to == 8+b)){
	return 12+b;
     } else
    if((from == 8+b) && (to==7+b)){
	return 13;
     } else
    if((from ==7+b) && (to == 5+b)){
	return 10;
     } else
    if((from == 5+b) && (to == 3+b)){
	return 6;
     } else
    if((from == 3+b) && ( to == 1+b)) {
	return 2;
     } else
    if ((from == 3+b) && (to == 2+b)) {
	return 3;
     } else
    if((from == 2+b) && (to == 5+b)){
	return 5;
     } else
    if((from == 5+b) && (to == 4+b)){
	return 7;
     } else
    if((from == 7+b) && (to == 6+b)){
	return 11;
     } else
    if((from==6+b) && (to==5+b)){
	return 9;
     } else {
      return 0;
     }
};



