#include "polygon.h"
// Gregory Ecker
// Graphics Programming
// Prof. Velho
// version 1.5  revised 7/19
// version 1.5a revision 7/21 to include line clipping using the
//	Sutherland-Cohen line clipping algorithm
// version 1.5b revision 7/24 Sutherland-Cohen polygon clip added.
// version 1.5c beta polygon clip revised.

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// polygon member functions defined :

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// internal list clean-up function.
void polygon::delete_point_list(point* first){
      point *tmp=first,*dead_node;
	while(tmp->Next() != NULL){
		dead_node=tmp;
		tmp=tmp->Next();
		delete dead_node;
	     };
	     delete tmp;
};

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// shell for 'polygon_clip', applies clip algorithm against each
// clip edge
void polygon::Clip(void){
// This call should be made only on a copy of the polygon data(for viewing only).
// clip polygon against all 4 sides of clip rectangle:
// *this object is changed irrevocably.
    vp=polygon_clip(vp,Right);
    vp=polygon_clip(vp,Bottom);
    vp=polygon_clip(vp,Left);
    vp=polygon_clip(vp,Top);
					//return clipped_poly;
};

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// polygon_clip: to be called by 'Clip'.
// returns a point list clipped against clipXmax,clipXmin,clipYmax,clipYmin
point* polygon::polygon_clip(point* Inpoly,ClipLine& clipBoundary){
  // list to hold new vertices.
    point* outPoly = new point();  // empty node at head of point list.
    point  *end=outPoly;    // end points to last point of OutPoly.

    point  p;
    point  *i,*s;

// start at top of list.
    trav=Inpoly;         //->Head();
// set s to point to last vertex in list.
    s=Inpoly;
    while(s->Next()!=NULL){   //  s=Inpoly->Last();
	s=s->Next();
      };


    while(trav->Next()!=NULL){	//trav!=NULL
	 p=*trav;
	 if(Inside(p,clipBoundary)==TRUE){
	    if(Inside(*s, clipBoundary)==TRUE){
		point *tmp=new point(p);
		tmp->SetNext(NULL);
		end->SetNext(tmp);
		end=end->Next();
			      // outPoly->Insert(*tmp);
	     } else
	       {
		i=new point();
		*i=Intersect(*s, p, clipBoundary);
		end->SetNext(i);
		end=end->Next();
					//	outPoly->Insert(*i);
		point *tmp=new point(p);
		end->SetNext(tmp);
		end=end->Next();
		end->SetNext(NULL);
					//	outPoly->Insert(*tmp);
	       }  // end inner else
	   } // end if
	 else
	   if (Inside(*s,clipBoundary) == TRUE){
		i=new point();
		*i=Intersect(*s,p, clipBoundary);
		end->SetNext(i);
		end=end->Next();
		end->SetNext(NULL);
					//	outPoly->Insert(*i);
	       }
	    s=&p;
	    trav=trav->Next();
     }   // end for
 // DBUG:
     delete_point_list(Inpoly);
 // ENDDEBUG
   // delete dead_node at head of outpoly list
	point *tmp=outPoly;
	outPoly=outPoly->Next();
	delete tmp;
     return outPoly;
 };

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// Draw : draws all the lines in the polygon.
//	  ASSUMES : projection matrix has been applied.
//	  DEPENDS : on a line function of the form : line(x1,y1,x2,y2);.

void polygon::Draw(void){
      point *left=vp;
      float leftx=left->x(); // save first point's coord.s
      float lefty=left->y(); // to connect with last point.
 //     float leftz=left->z();

	trav=left;         // start both endpoints at top of list.
	trav=trav->Next(); // move down list.

 // DBUG:
 //	int lx=left->x()+Tx;
 //	int rx=trav->x()+Tx;
 // ENDDBUG

	::line(left->x()+Tx,left->y()+Ty,trav->x()+Tx,trav->y()+Ty);
	while(trav->Next() != NULL){
		left=trav;       // get two new points.
		trav=trav->Next();
	::line(left->x()+Tx,left->y()+Ty,trav->x()+Tx,trav->y()+Ty);
	     };
	::line(trav->x()+Tx,trav->y()+Ty, leftx+Tx,lefty+Ty);
};

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// polygon copy constructor:

polygon::polygon(polygon& poly){
	numpts=poly.Numpts();
	point *tmp;
	point *external=poly.Head(); // point to external list's top.
	vp = new point(*external);  // create a copy of the first node.
	external=external->Next();
	trav = new point(*external);
	vp->SetNext(trav);
	external=external->Next();

     // copy each point in external polygon:
	while(external!= NULL){     // external -> Next();
	    tmp=new point(*external);
	    trav->SetNext(tmp);
	    external= external->Next();
	    trav=tmp;
	  };
	last=trav;		// set 'last' pointer
	trav->SetNext(NULL);

    // copy all public data:
	Tx=poly.Tx;  Ty=poly.Ty;
    // copy clip rectangle:
//	SetClipRect(poly.clipXmin,poly.clipYmax,poly.clipXmax,poly.clipYmin);
};

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// class destructor:
polygon::~polygon(){
    // delete all points allocated with 'new'
    point* dead_node;
    // start at top of list:
    trav=vp;
    while(trav != NULL){
	dead_node=trav;
	trav=trav->Next();
	delete dead_node;
      }
};

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// Inside:: tests a point to see if it is within clip bounds.

BOOLEAN polygon::Inside(point& testpoint,ClipLine& clipBoundary){
	BOOLEAN retval=FALSE;
       // Tx,Ty added for screen coordinate compatibility
	float x=testpoint.x()+Tx;
	float y=testpoint.y()+Ty;
assert((clipBoundary.clipExtreme==TOP)||(clipBoundary.clipExtreme==BOTTOM)||
(clipBoundary.clipExtreme==LEFT)||(clipBoundary.clipExtreme==RIGHT));

      switch (clipBoundary.clipExtreme) {
       case TOP : { if( y > clipBoundary.p0(1) ){
		     retval=TRUE; } break; }
       case BOTTOM : { if(y < clipBoundary.p0(1) ){
		     retval=TRUE; } break; }
       case LEFT   : { if(x > clipBoundary.p0(0) ){
		     retval=TRUE; } break; }
       case RIGHT  : { if(x < clipBoundary.p0(0) ){
		     retval=TRUE; } break; }
       }  // endswitch
  return retval;
};

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// applies matrix before drawing points.

void polygon::Apply_and_Draw(matrix& m){
    point* left;
     left=vp;
     trav=left->Next();
     left->Apply(m);
     trav->Apply(m);
 // apply matrix to all points.
    while(trav->Next() != NULL){
	left=trav;
	trav=trav->Next();
	trav->Apply(m);
      };
    Draw();
};

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// applies projection to each point in vertex list,
// ASSUMES working on a copy of real polygon data.
// DEPENDS on #define VIEW_DISTANCE to determine aperture of virtual lens.

void polygon::Project(void){
    float x,y,z;

    trav=vp; // start at top of list.
while(trav != NULL){
   // get current point's x,y values.
     x=trav->x();
     y=trav->y();
     z=trav->z();
    // divide x,y values * VIEW_DISTANCE by z value - VIEW_DISTANCE
    // check for divide by 0 :
    assert((z-VIEW_DISTANCE) != 0);
     x=(x*VIEW_DISTANCE)/(z-VIEW_DISTANCE);
     y=(y*VIEW_DISTANCE)/(z-VIEW_DISTANCE);
     trav->operator[](0)=-x;
     trav->operator[](1)=-y;
     trav=trav->Next();
  };
};

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// CompOutCode : computes code corresponding to clip rectangle and
//	endpoint given.
unsigned char polygon::CompOutCode(const vector& pt) const {
	unsigned char retcode=0;
       // check y coord. against y extremes of clip rect.
	if(pt[1] > clipYmax ){  // if y > ymax then code := top
	  retcode=TOP;
	 } else
	 if(pt[1] < clipYmin ){ // else if y < ymin then code := bottom
	   retcode = BOTTOM;
	  };
       // check x coord. against x extremes of clip rect.
	if(pt[0] > clipXmax ){  // if x > xmax then code+=RIGHT
	   retcode &= RIGHT;
	  } else
	 if(pt[0] < clipXmin ){ // if x < xmin then code+=LEFT
	   retcode &= LEFT;
	  };
       return(retcode);
};

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// applies matrix to each point in point list.

void polygon::Apply(matrix& m){
	trav=vp;	        // start at top of list.

     while(trav->Next() != NULL){  // apply matrix to each point in list.
	trav->Apply(m);
	trav=trav->Next();
       };
     trav->Apply(m);	// apply to last point.
};
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// applies matrix to each point in point list.
// calculates projection matrix for each point

void polygon::Apply_projection(matrix& m){
	trav=vp;	        // start at top of list.

     while(trav->Next() != NULL){  // apply matrix to each point in list.
	m.Project(trav->z());
	trav->Apply(m);
	trav=trav->Next();
       };
     m.Project(trav->z());
     trav->Apply(m);	// apply to last point.
};



////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// insert :  inserts point at end of point list in polygon.
//	and returns draw id.

int polygon::Insert(point& p){
   // special case, inserting into empty list.
   point* tmp;
    if(numpts==0){
      vp=new point(p);	// create new node with existing coord.s
      last=vp;
      last->SetId(0);	// set draw code.
      last->SetNext(NULL);
     } else	        // otherwise add new node onto end of list .
     {
      tmp = new point(p);
      last->SetNext(tmp);
      last=tmp;
      last->SetId(numpts);
      last->SetNext(NULL);
     } // end else
     numpts++;  // increment count of points.
     return (numpts-1);   // return the new point's draw id.
};
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// operator [], returns 'i'th component of polygon point list.
point& polygon::operator[](int i){
       int cur=0;
       assert(i>=0);
       assert(i<numpts);
	trav=vp;
      while(cur < i){
	trav=trav->Next();
	cur++;
       }
       return (*trav);
};

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// SetClipRect :
//	defines clipping rectangle by the coordinates of it's
//	upper-left corner and it's lower right corner.

void polygon::SetClipRect(float uplx,float uply,float dnrx,float dnry){
	// set internal float vars:
	 clipXmin=uplx;   clipYmin=dnry;
	 clipXmax=dnrx;   clipYmax=uply;

	// set internal vector rep of corners of clip rectangle.
	upleft[0]=clipXmin;   dnleft[0]=clipXmin;
	upleft[1]=clipYmax;   dnleft[1]=clipYmin;
	upleft[2]=0;          dnleft[2]=0;
	upleft[3]=1;	      dnleft[3]=1;

	upright[0]=clipXmax;  dnright[0]=clipXmax;
	upright[1]=clipYmax;  dnright[1]=clipYmin;
	upright[2]=0;	      dnright[2]=0;
	upright[3]=1;	      dnright[3]=1;

  // init 4 edges of clip rectangle:
	 Left.Initialize(upleft,dnleft); Left.clipExtreme=LEFT;
	 Right.Initialize(upright,dnright); Right.clipExtreme=RIGHT;
	 Bottom.Initialize(dnleft,dnright); Bottom.clipExtreme=BOTTOM;
	 Top.Initialize(upleft,upright);    Top.clipExtreme=TOP;
};
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// Intersect, calculates intersection point of line with clipBoundary:

point polygon::Intersect(point first,point second,ClipLine edge){
	float x,y;
	float x0=first.x();
	float x1=second.x();
	float y0=first.y();
	float y1=second.y();

   switch(edge.clipExtreme){
     case LEFT : { y=y0+(y1-y0)*(edge.p0(0)-x0)/(x1-x0);
		   x=edge.p0(0);
		   break; }
     case RIGHT : { y=y0+(y1-y0)*(edge.p0(0)-x0)/(x1-x0);
		    x=edge.p0(0);
		    break; }
     case BOTTOM : { x=x0+(x1-x0)*(edge.p0(1) - y0)/(y1-y0);
		     y=edge.p0(1);
		     break; }
     case TOP	 :  { x=x0+(x1-x0)*(edge.p0(1) - y0)/(y1-y0);
		      y=edge.p0(1);
		      break; }
	} // endswitch
    point intersection(x,y,0,1);
    return intersection;
};  // end Intersect


