// POLYGON.H
// Gregory Ecker
// Graphics Programming
// Prof. Velho
// Header file for POLYGON Class ver. 1.5a (new revision).
// extended 7/21 to include line clipping against a rectangle
//   defined by the float components clipXmax,clipXmin,clipYmax,clipYmin
//   1.5c beta
//   1.60 working

#ifndef _POLYGON_H
#define _POLYGON_H

#include "commdefs.h"	        // common definitions for all classes.
#include "point.h"
#include "clipline.h"	   	// for line clipping function.
#include <graphics.h>		// for line drawing function.
#include <stdlib.h> 		// for NULL constant def.
#include <assert.h>	        // for bounds checking

  // polygon is drawn with lines connecting each point,
  // and last point is connected to first.

class polygon {
friend ClipLine;
      public:
 // polygon clip and helping functions:
 inline void Clip(void);
	point Intersect(point first,point second,ClipLine edge);
	BOOLEAN Inside(point&,ClipLine&);
	point* polygon_clip(point*,ClipLine&);

// projection:
void	Project(void);
void delete_point_list(point* first);
// constructors:
	polygon(){ numpts=0; 	 // empty list.
		   vp = NULL;
		   last=NULL; }; // default constructor.
inline	polygon(polygon&);       // copy constructor.

void    SetClipRect(float,float,float,float);

// information-providing functions:
inline int Numpts(void) const { return numpts; };
//inline point Last(void) const { return *last; };
inline point* Last(void) const { return last; };
inline point* Head(void) const { return vp; };  // for copying op.s

 // polygon alterations:
inline void  Apply(matrix&);  // applies matrix to each point in point list.
inline int   Insert(point&);  // inserts new node at end of list., returns draw id.
void Apply_projection(matrix& m);

inline void Apply_and_Draw(matrix&); // applies matrix while drawing polygon.
inline void Draw(void);      // draws all lines in polygon.

unsigned char CompOutCode(const vector&) const; // computes spatial code for
						 // endpoint.
// member operator functions:
   point& operator[](int);
// destructor
  ~polygon();
// public data :
 float Tx,Ty;	             // translation factors to screen coordinates.
 float clipXmax,clipXmin;    // clipping rectangle defined by these 4 coord.s
 float clipYmax,clipYmin;
// four corners of clip rectangle:
    vector upleft,dnleft,upright,dnright;
// 4 edges of clip rectangle:
    ClipLine Left,Right,Bottom,Top;

// private data :
   private:
	 point  *vp,*trav,*last;  // pointers to manage list of vectors
				  // vp points to top of list, last points
				  // to bottom, trav is a temporary.
	 int numpts;              // number of points in polygon.
 };
#endif

